@@ -1,24 +1,23 @@
-import extend from 'lodash/extend';
+import merge from 'lodash/merge';
 import { getTheme } from '@grafana/ui';
-import { GrafanaTheme, GrafanaThemeType, PanelPluginMeta, DataSourceInstanceSettings } from '@grafana/data';
+import {
+  BuildInfo,
+  DataSourceInstanceSettings,
+  FeatureToggles,
+  GrafanaConfig,
+  GrafanaTheme,
+  GrafanaThemeType,
+  LicenseInfo,
+  PanelPluginMeta,
+  systemDateFormats,
+  SystemDateFormatSettings,
+} from '@grafana/data';
 
-export interface BuildInfo {
-  version: string;
-  commit: string;
-  isEnterprise: boolean;
-  env: string;
-  latestVersion: string;
-  hasUpdate: boolean;
-}
-
-interface FeatureToggles {
-  transformations: boolean;
-  inspect: boolean;
-  expressions: boolean;
-}
-export class GrafanaBootConfig {
+export class GrafanaBootConfig implements GrafanaConfig {
   datasources: { [str: string]: DataSourceInstanceSettings } = {};
   panels: { [key: string]: PanelPluginMeta } = {};
+  minRefreshInterval = '';
+  appUrl = '';
   appSubUrl = '';
   windowTitlePrefix = '';
   buildInfo: BuildInfo = {} as BuildInfo;
@@ -33,25 +32,35 @@ export class GrafanaBootConfig {
   alertingEnabled = false;
   alertingErrorOrTimeout = '';
   alertingNoDataOrNullValues = '';
+  alertingMinInterval = 1;
   authProxyEnabled = false;
   exploreEnabled = false;
   ldapEnabled = false;
   samlEnabled = false;
+  autoAssignOrg = true;
+  verifyEmailEnabled = false;
   oauth: any;
   disableUserSignUp = false;
   loginHint: any;
   passwordHint: any;
   loginError: any;
+  navTree: any;
   viewersCanEdit = false;
   editorsCanAdmin = false;
   disableSanitizeHtml = false;
   theme: GrafanaTheme;
   pluginsToPreload: string[] = [];
   featureToggles: FeatureToggles = {
-    transformations: false,
-    inspect: false,
+    live: false,
     expressions: false,
+    meta: false,
+    datasourceInsights: false,
+    reportGrid: false,
   };
+  licenseInfo: LicenseInfo = {} as LicenseInfo;
+  rendererAvailable = false;
+  http2Enabled = false;
+  dateFormats?: SystemDateFormatSettings;
 
   constructor(options: GrafanaBootConfig) {
     this.theme = options.bootData.user.lightTheme ? getTheme(GrafanaThemeType.Light) : getTheme(GrafanaThemeType.Dark);
@@ -63,6 +72,7 @@ export class GrafanaBootConfig {
       newPanelTitle: 'Panel Title',
       playlist_timespan: '1m',
       unsaved_changes_warning: true,
+      appUrl: '',
       appSubUrl: '',
       buildInfo: {
         version: 'v1.0',
@@ -75,16 +85,26 @@ export class GrafanaBootConfig {
       disableSanitizeHtml: false,
     };
 
-    extend(this, defaults, options);
+    merge(this, defaults, options);
+
+    if (this.dateFormats) {
+      systemDateFormats.update(this.dateFormats);
+    }
   }
 }
 
 const bootData = (window as any).grafanaBootData || {
   settings: {},
   user: {},
+  navTree: [],
 };
 
 const options = bootData.settings;
 options.bootData = bootData;
 
+/**
+ * Use this to access the {@link GrafanaBootConfig} for the current running Grafana instance.
+ *
+ * @public
+ */
 export const config = new GrafanaBootConfig(options);
